//==============================================================================
// dsMoveRouteExMZ.js
// Copyright (c) 2015 - 2020 DOURAKU
// Released under the MIT License.
// http://opensource.org/licenses/mit-license.php
//==============================================================================

/*:
 * @target MZ
 * @plugindesc 移動ルート拡張プラグイン ver1.0.0
 * @author 道楽
 * @base PluginCommonBase
 * @orderAfter PluginCommonBase
 *
 * @command SetOverrideEventId
 * @text コマンドの対象変更
 * @desc 特定のコマンドで「このイベント」を対象とするイベントIDを上書きします
 *       「並列実行」イベントには効果がありません
 *
 * @arg eventId
 * @type string
 * @default 0
 * @text イベントID
 * @desc 対象とするイベントID
 *       \v[変数ID]で変数を指定できます
 *
 * @command ResetOverrideEventId
 * @text コマンドの対象変更を解除
 * @desc 特定のコマンドのイベントID上書きを解除します
 *
 * @help
 * 「移動ルートの設定」で使用できる「スクリプト」を拡張します。
 *  有効なコマンド一覧は以下の通りです。
 *
 * -----------------------------------------------------------------------------
 * 使用できるスクリプトコマンド
 * ○ 特定の方向のシェイク
 *  ・縦シェイク
 *  SHAKE_V [シェイクの強さ] [シェイク速度] [持続時間]
 *  ・縦シェイク
 *  SHAKE_H [シェイクの強さ] [シェイク速度] [持続時間]
 * [シェイクの強さ] - 移動を繰り返す回数 (数字)
 * [シェイク速度]   - シェイクの速さ (数字)
 * [持続時間]       - シェイクの持続フレーム数 (数字)
 *
 * 例: SHAKE_V 5 5 30
 *     SHAKE_H 2 8 30
 * 
 * ○ 特定の方向へのノックバック
 *  ・左ノックバック
 *  KNOCKBACK_L [ノックバックの強さ] [ノックバック速度]
 *  ・右ノックバック
 *  KNOCKBACK_R [ノックバックの強さ] [ノックバック速度]
 *  ・上ノックバック
 *  KNOCKBACK_U [ノックバックの強さ] [ノックバック速度]
 *  ・下ノックバック
 *  KNOCKBACK_D [ノックバックの強さ] [ノックバック速度]
 * [ノックバックの強さ] - ノックバックする距離 (数字)
 * [ノックバック速度]   - 往復の速さ (数字)
 *
 * 例: KNOCKBACK_L 2 8
 *     KNOCKBACK_U 5 8
 *
 * ○ フェードイン・アウト
 *  ・フェードイン
 *  FADE_IN [所要時間] [ウェイトフラグ]
 *  ・フェードアウト
 *  FADE_OUT [所要時間] [ウェイトフラグ]
 * [所要時間]       - フェードに必要なフレーム数 (数字)
 * [ウェイトフラグ] - 次のコマンドの実行を待つかフラグ (true / false)
 * ※ウェイトフラグに「false」を指定し、次に移動コマンドを指定していると、
 *   フェードしながら移動ということが実現できます
 *
 * 例: FADE_IN 30 true
 *     FADE_OUT 60 false
 *
 * ○ キャラクター画像を指定したアクターの画像に変更
 *  ACTOR_IMG [アクターID]
 * [アクターID] - 変更する画像が設定されたアクターID (数字)
 *
 * 例: ACTOR_IMG 2
 *
 * ○ キャラクター画像を指定したメンバーの画像に変更
 *  MEMBER_IMG [隊列番号]
 * [隊列番号] - 変更する画像が設定されたメンバーの隊列番号 (数字)
 *              先頭のメンバーを「0」とする
 * ※隊列の人数よりも大きい番号を指定した場合は画像が変更されません
 *
 * 例: MEMBER_IMG 0
 *
 * ○ 目標位置への移動
 *  TOWARD_TARGET [X座標] [Y座標]
 * [X座標] - マップ上のX座標 (数字)
 * [Y座標] - マップ上のY座標 (数字)
 * ※距離が遠い場合は移動に失敗することがあります
 *
 * 例: TOWARD_TARGET 20 13
 *
 * ○ このイベントに向く
 *  TURN_TOWARD_THIS_EVENT
 *
 * 例: TURN_TOWARD_THIS_EVENT
 * 
 * ○ 向き設定
 *  SET_DIRECTION [向き]
 * [向き] - 向きを表す数値 (数字)
 *          2 = 下、4 = 左、6 = 右、8 = 上
 *
 * 例: SET_DIRECTION 2
 * 
 * ○ プライオリティ設定
 *  SET_PRIORITY [プライオリティ]
 * [プライオリティ] - プライオリティを表す数値 (数字)
 *                    0 = 通常キャラの下、1 = 通常キャラと同じ、2 = 通常キャラの上
 *
 * 例: SET_PRIORITY 2
 * 
 * ○ アニメーションパターン固定設定
 *  SET_FIXED_PATTERN [パターン]
 * [パターン] - アニメーションパターンを表す数値 (数字)
 *              0 = 左コマ、1 = 中コマ、2 = 右コマ
 *
 * 例: SET_FIXED_PATTERN 1
 * 
 * ○ アニメーションパターン固定設定解除
 *  RESET_FIXED_PATTERN
 *
 * 例: RESET_FIXED_PATTERN
 *
 * ○ 指定した範囲内でのランダムなウェイト
 *  WAIT_RANDOM [最小時間] [最大時間]
 * [最小時間] - ウェイトする最小フレーム数 (数字)
 * [最大時間] - ウェイトする最大フレーム数 (数字)
 *
 * 例: WAIT_RANDOM 30 60
 *
 * ○ 変数を指定したウェイト
 *  WAIT_VARIABLE [変数番号]
 * [変数番号] - ウェイト時間を指定する変数番号 (数字)
 *
 * 例: WAIT_VARIABLE 1
 */

var Imported = Imported || {};
Imported.dsMoveRouteExMZ = true;

(function (exports) {
	'use strict';

	//--------------------------------------------------------------------------
	/** PluginManager */
	const pluginName = "dsMoveRouteExMZ";

	PluginManager.registerCommand(pluginName, "SetOverrideEventId", args => {
		const eventId = Number(PluginManagerEx.convertEscapeCharacters(args.eventId));
		$gameMap.setOverrideEventId(eventId);
	});

	PluginManager.registerCommand(pluginName, "ResetOverrideEventId", args => {
		$gameMap.setOverrideEventId(0);
	});

	//--------------------------------------------------------------------------
	/** Game_CharacterBase */
	Game_CharacterBase.SHAKE_VERTICAL = 0;
	Game_CharacterBase.SHAKE_HORIZONTAL = 1;

	Game_CharacterBase.FADE_IN = 0;
	Game_CharacterBase.FADE_OUT = 1;

	(function () {
		const base = Game_CharacterBase.prototype.initMembers;
		Game_CharacterBase.prototype.initMembers = function()
		{
			base.apply(this, arguments);
			this.initShake();
			this.initFade();
		};
	}());

	Game_CharacterBase.prototype.initShake = function()
	{
		this._shakeType = Game_CharacterBase.SHAKE_VERTICAL;
		this._shakePower = 0;
		this._shakeSpeed = 0;
		this._shakeDuration = 0;
		this._shakeDirection = 1;
		this._shake = 0;
	};

	Game_CharacterBase.prototype.initFade = function()
	{
		this._fadeType = Game_CharacterBase.FADE_IN;
		this._fadeSpeed = 0;
		this._fadeDuration = 0;
		this._fadeWait = false;
	};

	(function () {
		const base = Game_CharacterBase.prototype.scrolledX;
		Game_CharacterBase.prototype.scrolledX = function()
		{
			const shake = (this._shakeType === Game_CharacterBase.SHAKE_HORIZONTAL) ? this._shake / $gameMap.tileWidth() : 0;
			return base.apply(this, arguments) + shake;
		};
	}());

	(function () {
		const base = Game_CharacterBase.prototype.scrolledY;
		Game_CharacterBase.prototype.scrolledY = function()
		{
			const shake = (this._shakeType === Game_CharacterBase.SHAKE_VERTICAL) ? this._shake / $gameMap.tileHeight() : 0;
			return base.apply(this, arguments) + shake;
		};
	}());

	Game_CharacterBase.prototype.setActorImage = function(actorId)
	{
		const actor = $gameActors.actor(actorId);
		if ( actor )
		{
			const characterName = actor ? actor.characterName() : '';
			const characterIndex = actor ? actor.characterIndex() : 0;
			this.setImage(characterName, characterIndex);
		}
	};

	Game_CharacterBase.prototype.setMemberImage = function(index)
	{
		const actor = $gameParty.members()[index];
		if ( actor )
		{
			const characterName = actor ? actor.characterName() : '';
			const characterIndex = actor ? actor.characterIndex() : 0;
			this.setImage(characterName, characterIndex);
		}
	};

	Game_CharacterBase.prototype.startShake = function(type, power, speed, duration, direction)
	{
		this._shakeType = type;
		this._shakePower = power;
		this._shakeSpeed = speed;
		this._shakeDuration = duration;
		this._shakeDirection = (direction || 1);
	};

	Game_CharacterBase.prototype.startKnockbackLeft = function(power, speed)
	{
		this.startShake(Game_CharacterBase.SHAKE_HORIZONTAL, power, speed, 1, -1);
	};

	Game_CharacterBase.prototype.startKnockbackRight = function(power, speed)
	{
		this.startShake(Game_CharacterBase.SHAKE_HORIZONTAL, power, speed, 1, 1);
	};

	Game_CharacterBase.prototype.startKnockbackUp = function(power, speed)
	{
		this.startShake(Game_CharacterBase.SHAKE_VERTICAL, power, speed, 1, -1);
	};

	Game_CharacterBase.prototype.startKnockbackDown = function(power, speed)
	{
		this.startShake(Game_CharacterBase.SHAKE_VERTICAL, power, speed, 1, 1);
	};

	Game_CharacterBase.prototype.startFade = function(type, duration, wait)
	{
		this._fadeType = type;
		this._fadeSpeed = 255 / duration;
		this._fadeDuration = duration;
		this._fadeWait = wait;
	};

	(function () {
		const base = Game_CharacterBase.prototype.isStopping;
		Game_CharacterBase.prototype.isStopping = function()
		{
			if ( !base.apply(this, arguments) )
			{
				return false;
			}
			if ( this.isShaking() )
			{
				return false;
			}
			if ( this._fadeWait )
			{
				if ( this.isFading() )
				{
					return false;
				}
			}
			return true;
		};
	}());

	Game_CharacterBase.prototype.isShaking = function()
	{
		return this._shakeDuration > 0 || this._shake !== 0;
	};

	Game_CharacterBase.prototype.isFading = function()
	{
		return this._fadeDuration > 0;
	};

	Game_CharacterBase.prototype.update = function()
	{
		if ( this.isStopping() )
		{
			this.updateStop();
		}
		if ( this.isShaking() )
		{
			this.updateShake();
		}
		else if ( this.isJumping() )
		{
			this.updateJump();
		}
		else if ( this.isMoving() )
		{
			this.updateMove();
		}
		this.updateAnimation();
		this.updateFade();
	};

	Game_CharacterBase.prototype.updateShake = function()
	{
		if ( this.isShaking() )
		{
			const delta = (this._shakePower * this._shakeSpeed * this._shakeDirection) / 10;
			if ( this._shakeDuration <= 1 && this._shake * (this._shake + delta) < 0 )
			{
				this._shake = 0;
			}
			else
			{
				this._shake += delta;
			}
			if ( this._shake > +this._shakePower * 2 )
			{
				this._shakeDirection = -1;
			}
			if ( this._shake < -this._shakePower * 2 )
			{
				this._shakeDirection = 1;
			}
			this._shakeDuration--;
		}
	};

	Game_CharacterBase.prototype.updateFade = function()
	{
		if ( this.isFading() )
		{
			switch ( this._fadeType )
			{
			case Game_CharacterBase.FADE_IN:
				this.setOpacity(this.opacity() + this._fadeSpeed);
				break;
			case Game_CharacterBase.FADE_OUT:
				this.setOpacity(this.opacity() - this._fadeSpeed);
				break;
			}
			this._fadeDuration--;
		}
	};

	//--------------------------------------------------------------------------
	/** Game_Character */
	(function () {
		const base = Game_Character.prototype.initMembers;
		Game_Character.prototype.initMembers = function()
		{
			base.apply(this, arguments);
			this._keepMoveRoute = false;
			this._fixedPattern = -1;
		};
	}());

	(function () {
		const base = Game_Character.prototype.setMoveRoute;
		Game_Character.prototype.setMoveRoute = function(moveRoute)
		{
			var moveRouteEx = moveRoute ? this.compileMoveRouteEx(moveRoute) : moveRoute;
			base.call(this, moveRouteEx);
		};
	}());

	(function () {
		const base = Game_Character.prototype.forceMoveRoute;
		Game_Character.prototype.forceMoveRoute = function(moveRoute)
		{
			var moveRouteEx = moveRoute ? this.compileMoveRouteEx(moveRoute) : moveRoute;
			base.call(this, moveRouteEx);
		};
	}());

	Game_Character.prototype.compileMoveRouteEx = function(moveRoute)
	{
		var moveRouteEx = {};
		moveRouteEx.list = [];
		moveRouteEx.repeat = moveRoute.repeat;
		moveRouteEx.skippable = moveRoute.skippable;
		moveRouteEx.wait = moveRoute.wait;
		moveRoute.list.forEach(function(command) {
			let commandClone = JSON.parse(JSON.stringify(command));
			if ( commandClone.code === Game_Character.ROUTE_SCRIPT )
			{
				var parameters = PluginManagerEx.convertEscapeCharacters(commandClone.parameters[0]);
				var s = parameters.split(' ');
				var cmd = s[0].toUpperCase();
				     if ( cmd === 'SHAKE_V' )                { commandClone.parameters[0] = 'this.startShake(Game_CharacterBase.SHAKE_VERTICAL,'+s[1]+','+s[2]+','+s[3]+', 1)'; }
				else if ( cmd === 'SHAKE_H' )                { commandClone.parameters[0] = 'this.startShake(Game_CharacterBase.SHAKE_HORIZONTAL,'+s[1]+','+s[2]+','+s[3]+', 1)'; }
				else if ( cmd === 'KNOCKBACK_L' )            { commandClone.parameters[0] = 'this.startKnockbackLeft('+s[1]+','+s[2]+')'; }
				else if ( cmd === 'KNOCKBACK_R' )            { commandClone.parameters[0] = 'this.startKnockbackRight('+s[1]+','+s[2]+')'; }
				else if ( cmd === 'KNOCKBACK_U' )            { commandClone.parameters[0] = 'this.startKnockbackUp('+s[1]+','+s[2]+')'; }
				else if ( cmd === 'KNOCKBACK_D' )            { commandClone.parameters[0] = 'this.startKnockbackDown('+s[1]+','+s[2]+')'; }
				else if ( cmd === 'FADE_IN' )                { commandClone.parameters[0] = 'this.startFade(Game_CharacterBase.FADE_IN,'+s[1]+','+s[2]+')'; }
				else if ( cmd === 'FADE_OUT' )               { commandClone.parameters[0] = 'this.startFade(Game_CharacterBase.FADE_OUT,'+s[1]+','+s[2]+')'; }
				else if ( cmd === 'ACTOR_IMG' )              { commandClone.parameters[0] = 'this.setActorImage('+s[1]+')'; }
				else if ( cmd === 'MEMBER_IMG' )             { commandClone.parameters[0] = 'this.setMemberImage('+s[1]+')'; }
				else if ( cmd === 'TOWARD_TARGET' )          { commandClone.parameters[0] = 'this.moveTowardTarget('+s[1]+','+s[2]+')'; }
				else if ( cmd === 'TURN_TOWARD_THIS_EVENT' ) { commandClone.parameters[0] = 'this.turnTowardThisEvent()'; }
				else if ( cmd === 'SET_DIRECTION' )          { commandClone.parameters[0] = 'this.setDirection('+s[1]+')'; }
				else if ( cmd === 'SET_PRIORITY' )           { commandClone.parameters[0] = 'this.setPriorityType('+s[1]+')'; }
				else if ( cmd === 'SET_FIXED_PATTERN' )      { commandClone.parameters[0] = 'this.setFixedPattern('+s[1]+')'; }
				else if ( cmd === 'RESET_FIXED_PATTERN' )    { commandClone.parameters[0] = 'this.resetFixedPattern()'; }
				else if ( cmd === 'WAIT_RANDOM' )            { commandClone.parameters[0] = 'this.waitRandom('+s[1]+','+s[2]+')'; }
				else if ( cmd === 'WAIT_VARIABLE' )          { commandClone.parameters[0] = 'this.waitVariable('+s[1]+')'; }
			}
			moveRouteEx.list.push(commandClone);
		}, this);
		return moveRouteEx;
	};

	(function () {
		const base = Game_Character.prototype.advanceMoveRouteIndex;
		Game_Character.prototype.advanceMoveRouteIndex = function()
		{
			if ( !this._keepMoveRoute )
			{
				base.call(this);
			}
			this._keepMoveRoute = false;
		};
	}());

	(function () {
		const base = Game_Character.prototype.updatePattern;
		Game_Character.prototype.updatePattern = function()
		{
			if ( this._fixedPattern >= 0 )
			{
				this._pattern = this._fixedPattern;
			}
			else
			{
				base.call(this);
			}
		};
	}());

	Game_Character.prototype.moveTowardTarget = function(x, y)
	{
		const direction = this.findDirectionTo(x, y);
		if ( direction > 0 )
		{
			this.moveStraight(direction);
			if ( this._x !== x || this._y !== y )
			{
				this._keepMoveRoute = true;
			}
		}
	};

	Game_Character.prototype.turnTowardThisEvent = function()
	{
		const event = $gameMap.runningEvent();
		if ( event )
		{
			this.turnTowardCharacter(event);
		}
	};

	Game_Character.prototype.waitRandom = function(waitMin, waitMax)
	{
		const waitRange = (waitMax - waitMin) + 1;
		this._waitCount = waitMin + Math.randomInt(waitRange) - 1;
	};

	Game_Character.prototype.waitVariable = function(variableId)
	{
		this._waitCount = $gameVariables.value(variableId) - 1;
	};

	Game_Character.prototype.setFixedPattern = function(pattern)
	{
		this._fixedPattern = pattern;
		this.setPattern(this._fixedPattern);
	};

	Game_Character.prototype.resetFixedPattern = function()
	{
		this._fixedPattern = -1;
	};

	//--------------------------------------------------------------------------
	/** Game_Map */
	Game_Map.prototype.setOverrideEventId = function(eventId)
	{
		this._interpreter.setOverrideEventId(eventId);
	};

	Game_Map.prototype.runningEvent = function()
	{
		if ( this._interpreter.isRunning() && this._interpreter.eventId() > 0 )
		{
			return this.event(this._interpreter.eventId());
		}
		return null;
	};

	//--------------------------------------------------------------------------
	/** Game_Interpreter */
	(function () {
		const base = Game_Interpreter.prototype.initialize;
		Game_Interpreter.prototype.initialize = function(depth)
		{
			base.apply(this, arguments);
			this._overrideEventId = 0;
		};
	}());

	(function () {
		const base = Game_Interpreter.prototype.setupChild;
		Game_Interpreter.prototype.setupChild = function(list, eventId)
		{
			base.apply(this, arguments);
			this._childInterpreter.setOverrideEventId(this._overrideEventId);
		};
	}());

	(function () {
		const base = Game_Interpreter.prototype.character;
		Game_Interpreter.prototype.character = function(param)
		{
			const overrideCharacter = this.overrideCharacter(param);
			if ( overrideCharacter )
			{
				return overrideCharacter;
			}
			return base.apply(this, arguments);
		};
	}());

	Game_Interpreter.prototype.overrideCharacter = function(param)
	{
		if ( !this.isOnCurrentMap() )
		{
			return null;
		}
		if ( param !== 0 )
		{
			return null;
		}
		return (this._overrideEventId > 0) ? $gameMap.event(this._overrideEventId) : null;
	};

	Game_Interpreter.prototype.setOverrideEventId = function(eventId)
	{
		this._overrideEventId = eventId;
		if ( this._childInterpreter )
		{
			this._childInterpreter.setOverrideEventId(this._overrideEventId);
		}
	};

}((this.dsMoveRouteExMZ = this.dsMoveRouteExMZ || {})));
